package lsedit;

/** 
 *   Developed by: Ian Davis in Summer 1997 for Grant Weddell
 *   Class which hides all of the various display options which may be passed (as a structure)
 *   to Layouter.   
 */
public class HiGraphLayout {

	public final int			horizVertexGap = 20;		// default horizontal gap between boxes	 
	public final int			vertVertexGap  = 20;		// default vertical gap between boxes
	public final int			horizMargin    = 2;			// horizontal margin for the graph
	public final int			vertMargin     = 2;			// vertical margin for graph

	/* Global display parameters */

	private int					m_xgap     = vertVertexGap;	// specify horizontal gap between nodes
	private int					m_ygap     = horizVertexGap;// specify vertical gap between nodes
	private int         		m_simplex  = 10000;			// specify the maximum number of simplex pivots
	private int					m_crossing = 25;			// specify the maximum number of crossing cycles
	private int					m_weight_1dummy = 0;		// Weight of edge with one dummy node in it (2 if showing such nodes)
	private int					m_weight_2dummy = 0;		// Weight of edge with two dummy nodes in it (8 if showing such nodes)
	private int					m_outside_bias  = 1;		// Bias for outside edges in layout
	private int					m_group_children = 0;		// Group children above/below higraphs

	/* Other */

	private String				m_font = "Helvetica";		// Font to use
	private int					m_fontsize = 14;
	private	boolean				m_bends    = false;			// True if will be using the bends in arcs
	
	/**
	 * Constructor.
	 */

	public HiGraphLayout() 
	{
	}

	public void setBends(boolean value)
	{
		if (value) {
			m_weight_1dummy = 2;
			m_weight_2dummy = 8;
		} else {
			m_weight_1dummy = 0;
			m_weight_2dummy = 0;
		}
		m_bends = value;
	}
	
	public boolean hasBends()
	{
		return m_bends;
	}

	public int	xgap() 
	{
		return (m_xgap);
	}

	public boolean xgap(int value) 
	{
		if (value >= 1) {
			m_xgap = value;
			return(true);
		}
		return(false);
	}

	public int ygap() 
	{
		return (m_ygap);
	}

	public boolean ygap(int value) 
	{
		if (value >= 1) {
			m_ygap = value;
			return(true);
		}
		return(false);
	}

	public int	simplex() 
	{
		return(m_simplex);
	}

	public boolean simplex(int value) 
	{
		if (value >= 0) {
			m_simplex = value;
			return(true);
		}
		return(false);
	}

	public int crossing() 
	{
		return(m_crossing);
	}

	public boolean crossing(int value) 
	{
		if (value >= 0) {
			m_crossing = value;
			return(true);
		}
		return(false);
	}

	public int weight_1dummy() 
	{
		return(m_weight_1dummy);
	}

	public boolean weight_1dummy(int value) 
	{
		if (value >= 1) {
			m_weight_1dummy = value;
			return(true);
		}
		return(false);
	}

	public int weight_2dummy() 
	{
		return(m_weight_2dummy);
	}

	public boolean weight_2dummy(int value) 
	{
		if (value >= 1) {
			m_weight_2dummy = value;
			return(true);
		}
		return(false);
	}

	public int outside_bias() 
	{
		return(m_outside_bias);
	}

	public boolean outside_bias(int value) 
	{
		if (value >= 0) {
			m_outside_bias = value;
			return(true);
		}
		return(false);
	}

	public int group_children() 
	{
		return(m_group_children);
	}

	public boolean group_children(int value) 
	{
		if (value >= -1 && value <= 1) {
			m_group_children = value;
			return(true);
		}
		return(false);
	}

	public String font() {
		return(m_font);
	}

	public void font(String choice) 
	{
		m_font = choice;
	}

	public int fontsize() 
	{
		return(m_fontsize);
	}

	public boolean fontsize(int value) 
	{
		if (value >= 1 && value <= 30) {
			m_fontsize = value;
			return(true);
		}
		return(false);
	}

	int xmargin() {
		return (horizMargin);
	}

	int ymargin() {
		return (vertMargin);
	}

	// --- Main processing logic

	// Once the highgraph has been laid out this may be called separately if the fonts change

	public boolean coordinates(HiGraph root) 
	{
		try {
//			System.out.println("Laying out");
			HiGraphCoordinates.coordinates(this, root);
//			System.out.println("Layout done");
			return(true);
		}
		catch (HiGraphException e) {
			System.out.println("HiGraph error in HiGraphCoordinates: " + e.getMessage());
			e.printStackTrace();
		}
		catch (Exception e) {
			System.out.println("Error during HiGraph layout in HiGraphCoordinates: " + e.getMessage());
			e.printStackTrace();
		}
		return(false);
	}

	public boolean layout(HiGraph root)
	{
		boolean ret;

		try {
			
//			System.out.println("HiGraphLayout: beginning group");
			HiGroup.group(root, group_children());

			// Compute an optimal vertical ranking 
//			System.out.println("HiGraphLayout: beginning ranking");
			HiRank.compute(root, simplex());

			// Compute a good horizontal ordering for higraphs based on edges between higraphs
//			System.out.println("HiGraphLayout: beginning child ordering");
			HiChildren.order(root, outside_bias());

			// Compute a good horizontal ordering within each higraph by exchanging positions
			// System.out.println("Before interchange\n");
			// root.dump();

//			System.out.println("HiGraphLayout: beginning row ordering");
			HiRow.order(this, root);
			// System.out.println("After interchange\n");
			// root.dump();
//			System.out.println("HiGraphLayout: beginning coordinates");
			ret = coordinates(root);
//			System.out.println("HiGraphLayout: success");
			return(ret);
		}
		catch (HiGraphException e) {
			System.out.println("HiGraph error in " + root + ": " + e.getMessage());
			e.printStackTrace();
		}
		catch (Exception e) {
			System.out.println("Error during HiGraph layout in " + root + ": " + e.getMessage());
			e.printStackTrace();
		}
		return(false);
	}
}

